@@ -65,9 +65,7 @@ query6 = '
 SELECT [A1CNBR] FROM [SAM].[dbo].[DiabetesClinicall]'
 
 test_that("SQL Server - Returns correct selected data in data frame", {
-  skip_on_cran()
-  skip_if_no_MSSQL(tableName = "HCRDiabetesClinical", 
-                   connString = connection.string)
+  skip_on_not_appveyor()
 
   expect_equal(selectData(MSSQLConnectionString = connection.string, query4),
                data.frame(SystolicBPNBR = c(167,153,170,187,188,
@@ -75,9 +73,7 @@ test_that("SQL Server - Returns correct selected data in data frame", {
 })
 
 test_that("SQL Server - Returns zero rows msg when zero rows selected", {
-  skip_on_cran()
-  skip_if_no_MSSQL(tableName = "HCRDiabetesClinical", 
-                   connString = connection.string)
+  skip_on_not_appveyor()
 
   expect_warning(grepl(selectData(MSSQLConnectionString = connection.string, 
                                   query5),
@@ -85,9 +81,7 @@ test_that("SQL Server - Returns zero rows msg when zero rows selected", {
 })
 
 test_that("SQL Server - Returns SQL error message when SQL error", {
-  skip_on_cran()
-  skip_if_no_MSSQL(tableName = "HCRDiabetesClinical", 
-                   connString = connection.string)
+  skip_on_not_appveyor()
 
   expect_error(selectData(MSSQLConnectionString = connection.string, query6),
                'Your SQL likely contains an error.')
